/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Attachment Extractor.
 *
 * The Initial Developer of the Original Code is
 * Andrew Williamson <eviljeff@eviljeff.com>.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Matteo F Zazzetta 
 * (bug fixes, contributed complete rewrite of validateFileName and supporting 
 *	functions to handle long filenames and inspired rewrite of cleanSubjectLine)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var aewindow= {
	currentTask:null,
	queuedTasks:new Array(),
	progress_tracker:null,
	attachmentextractor:null,
	messenger:null,
	aedump: {},
	preferenceService: null,
	
	filecomponent:Components.classes["@mozilla.org/file/local;1"],
    fileinterface:Components.interfaces.nsILocalFile,
	nsMsgViewCommandType:Components.interfaces.nsMsgViewCommandType,
	
	supportsarraycomponent:Components.classes['@mozilla.org/supports-array;1'],
	supportsarrayinterface:Components.interfaces.nsISupportsArray,
	
	makeFile:function() {return this.filecomponent.createInstance(this.fileinterface);},
	newnsIArray:function() {return this.supportsarraycomponent.createInstance(this.supportsarrayinterface);},
	AEMessage:{},
	
	inited:false
};

aewindow.init=function() {
    if (this.inited) return;
    else this.inited=true;
    aewindow.progress_tracker=progress_tracker;
    aewindow.attachmentextractor=attachmentextractor;
    aewindow.preferenceService=aewindow.attachmentextractor.preferenceService;
    aewindow.aedump=aedump;
    aewindow.AEMessage=AEMessage;
    window.onerror=aewindow.errorCatcher;
	var Cc=Components.classes;
    var Ci=Components.interfaces;
  
    aewindow.messenger = Cc["@mozilla.org/messenger;1"].createInstance().QueryInterface(Ci.nsIMessenger);
	aewindow.msgWindow = Cc["@mozilla.org/messenger/msgwindow;1"].createInstance().QueryInterface(Ci.nsIMsgWindow);
 	aewindow.mailSession = Cc["@mozilla.org/messenger/services/session;1"].getService(Ci.nsIMsgMailSession);
			
	if ("arguments" in window && window.arguments.length > 0) {
		aewindow.queueAETask(window.arguments[0],window.arguments[1],
							 window.arguments[2],window.arguments[3],
							 window.arguments[4],window.arguments[5]);
		aewindow.consumeAETask();
    } else window.close();
}
	
aewindow.uninit=function() {
	if (!aewindow.currentTask) return;
	aewindow.currentTask.tidyUp();
}

aewindow.errorCatcher=function(message,url,number) {
  try{
	aewindow.aedump("// error thrown: '"+message+"' @ line:"+number+" in "+url+"\n",0);
	var pwindow=(aewindow.progress_tracker)?aewindow.progress_tracker.getWindowByType("mail:3pane"):window;
	var bundle=aewindow.attachmentextractor.aeStringBundle;
	aewindow.attachmentextractor.promptService.alert(pwindow,
			bundle.GetStringFromName("GenericErrorDialog"),
			bundle.GetStringFromName("GenericErrorMessage"));
  }catch (e) {aewindow.aedump("// Error in the error catcher! : "+e+"\n",0);}
  window.close();
}	

aewindow.consumeAETask=function() {
	aewindow.aedump('{function:consumeAETask}\n',2);
	if (aewindow.currentTask) aewindow.currentTask.tidyUp();
	aewindow.currentTask=(aewindow.queuedTasks.length==0)? null : aewindow.queuedTasks.shift();
	if (!aewindow.currentTask) {
		window.close();
		return null;
	}
	if (!aewindow.currentTask.initialize()) return aewindow.consumeAETask();
	aewindow.aedump(aewindow.currentTask+"\n",2);
	aewindow.currentTask.start();
	return null;
};
	
aewindow.queueAETask=function(savefolder,messages,filenamepattern,originalView,background,justdelete) {
	aewindow.aedump('{function:queueAETask}\n',2);
	aewindow.queuedTasks.push(new AETask(savefolder,messages,filenamepattern,aewindow,originalView,background,justdelete));
}
	
/* ****************************** AETask ******************************************************** */

function AETask(savefolder,selectedURIs,filenamepattern,aewindow,originalView,isBackground,justDeleteAttachments) {
  //public vars
  /* this.membername=value */
  if (!filenamepattern) filenamepattern=aewindow.attachmentextractor.getPref("filenamepattern");
  this.filemaker=(justDeleteAttachments? null : new AttachmentFileMaker(filenamepattern,savefolder,aewindow));
  this.currentMessage=null;
  this.currentDisplayedURI=null;
  this.detachCancellationTimeout=null;
  this.listeningforMessageId="";
  
  //private vars
  /* var membername=value */
  var that=this;
  var Cc=Components.classes;
  var Ci=Components.interfaces;
  var Cr=Components.results;
  var currentviewpref=null;
  var downloadwindowpref=null;
  var downloadwindowdelay=null;
  var compactfolderspref=null;
  var currentindex=-1;
  var includeexcludearray=null;
  var initialized=false;
  var savedFiles=new Array();
  
  //protected methods
  function initialize() {
	if (initialized) return true;
	aewindow.aedump('{function:AETask.initialize}\n',2);
	if (!selectedURIs) {
		aewindow.aedump("// ae aborted because no messages selected.\n",0);
		return false;		
	}
	if (!selectedURIs && !justDeleteAttachments) {
		aewindow.aedump("// ae aborted because no save path passed on.\n",0);
		return false;		
	}
	if (isBackground&&justDeleteAttachments) {
		aewindow.aedump("// ae aborted you can't currently just delete attachments on auto-extract.\n",0);
		return false;		
	}
	if (this.doDetachConfirm() && !aewindow.attachmentextractor.promptService.confirm(
		aewindow.progress_tracker.getWindowByType("mail:3pane"),
		aewindow.attachmentextractor.aeStringBundle.GetStringFromName("ConfirmDetachDialogTitle"),
		aewindow.attachmentextractor.aeStringBundle.GetStringFromName("ConfirmDetachDialogMessage"))) {
			aewindow.aedump("// ae aborted detach confirmation cancelled.\n",1);
			return false;
	}
	window.MsgWindowCommands = new AEMsgWindowCommands(that); 
	// new aewindow.currentTask.nsMsgWindowCommands();  
	//aewindow.msgWindow.windowCommands = new nsMsgWindowCommands();//not in 1.8 branch, only trunk.
	
	if (originalView) {
	aewindow.gDBView = originalView.cloneDBView(aewindow.messenger, aewindow.msgWindow, new aeMsgDBViewCommandUpdater());
	}else {
		aewindow.gDBView = Cc["@mozilla.org/messenger/msgdbview;1?type=threaded"].createInstance(Ci.nsIMsgDBView);
		aewindow.gDBView.init(aewindow.messenger, aewindow.msgWindow, new aeMsgDBViewCommandUpdater());
	}
	aewindow.msgWindow.msgHeaderSink = that;
	aewindow.msgWindow.SetDOMWindow(window);
	aewindow.mailSession.AddMsgWindow(aewindow.msgWindow); 
	
	aewindow.messenger.SetWindow(document.getElementById("messagepane").contentWindow, aewindow.msgWindow);
	
	initialized=true;
	
    currentviewpref=aewindow.preferenceService.getIntPref("mailnews.display.html_as");
    downloadwindowpref=aewindow.preferenceService.getIntPref("browser.downloadmanager.behavior");
    downloadwindowdelay=aewindow.preferenceService.getIntPref("browser.download.manager.openDelay");
    compactfolderspref=aewindow.preferenceService.getBoolPref("mail.prompt_purge_threshhold");
	
	Cc["@mozilla.org/messenger/services/session;1"].getService(Ci.nsIMsgMailSession).AddFolderListener(that, Ci.nsIFolderListener.added);
	
	return true;
  }
  this.initialize=initialize;
  
  this.start = function() {
	aewindow.aedump('{function:AETask.start}\n',2);
	aewindow.progress_tracker.starting_extraction();
	
	if (getPref("extractinlinetoo")) {
		aewindow.preferenceService.setIntPref("mailnews.display.html_as",1);
	}
	aewindow.preferenceService.setIntPref("browser.downloadmanager.behavior",2);
	aewindow.preferenceService.setIntPref("browser.download.manager.openDelay",30000);
	aewindow.preferenceService.setBoolPref("mail.prompt_purge_threshhold",false);
	//
	that.active=true; 
	aewindow.progress_tracker.reset_tracker();
	this.selectNextMessage();
  };
  
  this.selectNextMessage = function() {
    //try {
	aewindow.aedump('{function:AETask.selectNextMessage}\n',2);
	if (currentindex>-1 && that.isEndLaunchEnabled()) that.storeSavedFiles();
	currentindex++;
	if (currentindex==selectedURIs.length) {
		that.endAttachmentextraction();
	}
	else {
      try{		
		var uri=selectedURIs[currentindex];
		that.currentDisplayedURI=uri;
		that.currentMessage=new aewindow.AEMessage(uri,currentindex,aewindow);
		aewindow.progress_tracker.starting_message(currentindex,selectedURIs.length);	
		var uriFolder=aewindow.messenger.msgHdrFromURI(uri).folder;
		//var changeFolder=isBackground;
		//var changeFolder=(aewindow.gDBView.searchSession==null)
		var changeFolder=true;
		try { 
			var a=aewindow.gDBView.QueryInterface(Ci.nsIMsgCopyServiceListener); 
			changeFolder=false;
		} catch (e) {aewindow.aedump("//ae: can safely change folder: "+changeFolder+"\n",3);}
		
		//aedump("// current Folder: "+aewindow.gDBView.msgFolder.name+"\n");
		if (changeFolder&&aewindow.gDBView.msgFolder!=uriFolder) {
			aewindow.aedump("// ae: folder different so open new folder\n",3);
			var msgDatabase = uriFolder.getMsgDatabase(aewindow.msgWindow);
			var sortType=(msgDatabase)?msgDatabase.dBFolderInfo.sortType:aewindow.gDBView.sortType;
			var sortOrder=(msgDatabase)?msgDatabase.dBFolderInfo.sortOrder:aewindow.gDBView.sortOrder;
			var viewFlags=(msgDatabase)?msgDatabase.dBFolderInfo.viewFlags:aewindow.gDBView.viewFlags;
			var count=new Object();
			aewindow.gDBView.open(uriFolder,sortType,sortOrder,viewFlags,count);
		} else aewindow.aedump("// ae: folder same so no need to open\n",3);
	
		var msgkey=aewindow.messenger.msgHdrFromURI(uri).messageKey;
		var vindex=aewindow.gDBView.findIndexFromKey(msgkey, true);
		aewindow.aedump("[uri: "+uri+"; view index: "+vindex+"; folder: "+uriFolder.name+"]\n",1);
		//aewindow.gDBView.selectMsgByKey(msgkey); //disabled 5/1/07 to fix del/det bug
		aewindow.gDBView.loadMessageByViewIndex(vindex);
	  }catch (e) {
	  	aewindow.aedump("// "+e+" thrown in selectNextMessage, skipping...",0);
		that.selectNextMessage();
	  }
		//aewindow.messenger.OpenURL(uri);	
	}
  /*}
	catch (e) {
        alert(e.name+": "+e.message);
    }*/
  };
    
  this.endAttachmentextraction = function() {
	aewindow.aedump('{function:AETask.endAttachmentextraction}\n',2);
	if (!that.active) return;
	that.active=false;
	
	if (that.isEndLaunchEnabled()) {
		var pref="attachmentextractor."+(isBackground?"autoextract":"actionafterextract")+".endlaunch.application";
		if (aewindow.preferenceService.prefHasUserValue(pref)&& savedFiles.length>0) {
		  try{ 
		    var exec=aewindow.preferenceService.getComplexValue(pref, Ci.nsILocalFile);
			var process=Cc["@mozilla.org/process/util;1"].createInstance(Ci.nsIProcess);
			aewindow.aedump('// Launching external App ... ',3);
			aewindow.aedump(' \''+exec.leafName+'\' with \''+savedFiles+'\'\n',3);
			process.init(exec);
			process.run(false,savedFiles,savedFiles.length);
		  } catch (e) {
			  aewindow.aedump(e);
		  }
		}
	}
	setTimeout(aewindow.consumeAETask,1);
	if (that.isNotifywhendoneEnabled()&&aewindow.queuedTasks.length==0) 
		aewindow.currentTask.alertCheck(
			aewindow.attachmentextractor.aeStringBundle.GetStringFromName("DoneExtractingDialogTitle"),
			aewindow.attachmentextractor.aeStringBundle.GetStringFromName("DoneExtractingDialogMessage"),
			aewindow.attachmentextractor.aeStringBundle.GetStringFromName("DoneExtractingDialogNotifyAgain"),
			'notifywhendone',
			true);
	//window.close();
  };
  
  this.tidyUp = function() {  
    if (!initialized) return;
	aewindow.aedump('{function:AETask.tidyUp}\n',2);
	if (getPref("extractinlinetoo")) {
		aewindow.preferenceService.setIntPref("mailnews.display.html_as",currentviewpref);
	}
	aewindow.preferenceService.setIntPref("browser.downloadmanager.behavior",downloadwindowpref);
	aewindow.preferenceService.setIntPref("browser.download.manager.openDelay",downloadwindowdelay);
	aewindow.preferenceService.setBoolPref("mail.prompt_purge_threshhold",compactfolderspref);
	
	Cc["@mozilla.org/messenger/services/session;1"].getService(Ci.nsIMsgMailSession).RemoveFolderListener(that);
	
	window.MsgWindowCommands=null;
	aewindow.msgWindow.msgHeaderSink=null;
	
	aewindow.messenger.SetWindow(null, aewindow.msgWindow);
	aewindow.mailSession.RemoveMsgWindow(aewindow.msgWindow); 
	
	aewindow.progress_tracker.ended_extraction();
  }
  
  this.alertCheck= function(titletext,messagetext,dontasktext,pref,prefvalue){
	var checkResult = {value:prefvalue};
	var pwindow=aewindow.progress_tracker.getWindowByType("mail:3pane");
	aewindow.attachmentextractor.promptService.alertCheck(pwindow,titletext,messagetext,dontasktext,checkResult);
	setPref(pref,checkResult.value);
  };
  
  /* to implement nsIMsgHeaderSink */
  this.handleAttachment= function(contentType, url, displayName, uri, isExternalAttachment) 
  {
	if (aewindow.progress_tracker.attachment_busy()||aewindow.progress_tracker.is_detaching()) return;
	aewindow.aedump('{function:AETask.handleAttachment}\n',2);  
	if (isExternalAttachment) {
		aewindow.aedump(displayName+" is external attachment so ignore\n",1);
		return;
	}
	if (!include_exclude_check2(displayName)) {
  		aewindow.aedump(displayName+" failed include/exclude filename check\n",1);
		return;
  	}
  	if (!mime_check(contentType)) {
		aewindow.aedump(displayName+" failed contentType check\n",1);
		return;
	}
	that.currentMessage.addAttachment(contentType,url,displayName.replace(/ +/g, " "),uri,isExternalAttachment); 
  };
    
  this.onEndAllAttachments= function() {
		if (!that.active) return;
		aewindow.aedump('{function:AETask.onEndAllAttachments}\n',2);
		if (aewindow.progress_tracker.is_detaching()) {
			//aewindow.aedump('//onEndAllAttachments\n',0);
			clearTimeout(that.detachCancellationTimeout);
			that.currentMessage.doDelete();
			return;
		}
		if ((!aewindow.progress_tracker.attachment_busy()) && 
			(selectedURIs[currentindex]==that.currentDisplayedURI)){ 
			that.currentMessage.attachmentextraction(selectedURIs.length);
		}
  };    

  this.processHeaders= function(headerNameEnumerator, headerValueEnumerator, dontCollectAddress) {};
  this.onEndMsgDownload= function(url) {};
  this.onEndMsgHeaders= function(url) {};
  this.onMsgHasRemoteContent= function(aMsgHdr) {};
  this.mDummyMsgHeader=null;
  this.getDummyMsgHeader=function() {
      if (!that.mDummyMsgHeader) that.mDummyMsgHeader = new nsDummyMsgHeader();
      return this.mDummyMsgHeader;
  };
    
  /* to implement nsIMsgWindowCommands */
  this.SelectFolder= function(folderUri) {};
  
  this.SelectMessage= function(messageUri) {
	 that.currentDisplayedURI=messageUri;
	 aewindow.aedump("{function:AETask.SelectMessage("+messageUri+")}\n");
	 var msgkey=aewindow.messenger.msgHdrFromURI(messageUri).messageKey;
	 aewindow.gDBView.loadMessageByViewIndex(aewindow.gDBView.findIndexFromKey(msgkey, true));
  };
    
  /* to implement nsIMsgMessagePaneController */
  this.clearMsgPane= function() {};  
  
  /* to implement nsIFolderListener */
  
  this.OnItemAdded= function(parentItem, item) {
	if (that.listeningforMessageId=="") return;
	var mail=item.QueryInterface(Components.interfaces.nsIMsgDBHdr);
    var folder=parentItem.QueryInterface(Components.interfaces.nsIMsgFolder); 
   	if (mail.messageId==that.listeningforMessageId) {
		var vindex=aewindow.gDBView.findIndexFromKey(mail.messageKey, true);
		if (vindex==0xFFFFFFFF) return;
		aedump("{function:AETask.OnItemAdded()}: view index: "+vindex+", oldURI: "+aewindow.currentTask.currentDisplayedURI+", newURI: "+folder.getUriForMsg(mail)+"\n",2);
		that.currentDisplayedURI=folder.getUriForMsg(mail);
		aewindow.gDBView.loadMessageByViewIndex(vindex);
	} else aedump("// 'wrong' msgid added: "+mail.messageId+", ignoring.\n",3);
  };
 
  /*OnItemPropertyChanged: function(item, property, oldValue, newValue) {},
  OnItemRemoved: function(parentItem, item) {},
  OnItemIntPropertyChanged: function(item, property, oldValue, newValue) {},
  OnItemBoolPropertyChanged: function(item, property, oldValue, newValue) {},
  OnItemUnicharPropertyChanged: function(item, property, oldValue, newValue){},
  OnItemPropertyFlagChanged: function(item, property, oldFlag, newFlag) {},
  OnItemEvent: function(folder, event) {}   */
  
  /* should support all interfaces we support */
  this.QueryInterface= function (iid)
  {
    if (iid.equals(Ci.nsIMsgWindowCommands) ||
		iid.equals(Ci.nsIMsgMessagePaneController) ||
        iid.equals(Ci.nsIMsgHeaderSink) ||
		iid.equals(Ci.nsIFolderListener) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_NOINTERFACE;
  };
  
  this.storeSavedFiles=function() {
	for (var i=0;i<that.currentMessage.getNumAttachments();i++) {
		if (that.currentMessage.getSavedFile(i)) savedFiles.push('"'+that.currentMessage.getSavedFile(i).path+'"');
	}
  };
  
  this.isExtractEnabled=function() {
	  return justDeleteAttachments? false : (isBackground || (getPref("extract.mode")!=-1)); 
  };
  
  this.isDeleteEnabled=function() {
	  return !justDeleteAttachments && getPref(isBackground?"autoextract.delete":"actionafterextract.delete");
  };
  
  this.isMarkreadEnabled=function() {
	  return !justDeleteAttachments && getPref(isBackground?"autoextract.markread":"actionafterextract.markread");
  };
  
  this.isSaveMessageEnabled=function() {
	  return !justDeleteAttachments && getPref(isBackground?"autoextract.savemessage":"actionafterextract.savemessage");
  };
  
  this.isLaunchEnabled=function() {
	  return !justDeleteAttachments && getPref(isBackground?"autoextract.launch":"actionafterextract.launch");
  };
  
  this.isDetachEnabled=function() {
	  return justDeleteAttachments || getPref(isBackground?"autoextract.detach":"actionafterextract.detach");
  };
  
  this.isCleartagEnabled=function() {
	  return isBackground && (getPref("autoextract.ontriggeronly") && getPref("autoextract.cleartag")) ;
  };
  
  this.isNotifywhendoneEnabled=function () {
	  return isBackground? false : (selectedURIs.length>1 && getPref("notifywhendone"));
  };
  
  this.isEndLaunchEnabled=function() {
	  return !justDeleteAttachments && getPref(isBackground?"autoextract.endlaunch":"actionafterextract.endlaunch");
  };
  
  this.getOverwritePolicy=function () {
	  return getPref(isBackground?"autoextract.overwritepolicy":"overwritepolicy");
  };
  
  this.getDetachMode=function() {
	  return justDeleteAttachments? 1 : getPref(isBackground?"autoextract.detach.mode":"actionafterextract.detach.mode");
  };
  
  this.doDetachConfirm=function() {
	  return !isBackground && that.isDetachEnabled() && (that.getDetachMode()!=0) && getPref("actionafterextract.detach.confirm");
  };
  
  this.nextUri=function() {
	  if (currentindex+1==selectedURIs.length) return null;
	  return selectedURIs[currentindex+1];
  };
  
  //private methods
  
  function mime_check(mimetype) {
	return (mimetype!="text/x-moz-deleted");
  };
  
  //   
  function strToReg(value,index,array) {
	  if (value.charAt(0)=='/') {
		  var expr=value.substring(1,value.lastIndexOf('/'));
		  var flags=value.substring(value.lastIndexOf('/')+1);
		  aewindow.aedump("//directly specificed reg-ex: /"+expr+"/"+flags+"\n",3);
		  return new RegExp(expr,flags);
	  }
	  value=value.replace(".","\\.");
	  value=value.replace("*",".*");
	  value="^"+value+"$";
	  return new RegExp(value,"i");
  }
  
  //   
  function include_exclude_check2(filename) {
	filename=filename.replace(/'/g,"\'");
	if (includeexcludearray==null) {
		if (getPref("includeenabled")==1) includeexcludearray=getPref("includepatterns4").split(';');
		else includeexcludearray=getPref("excludepatterns4").split(';');
		
		/*var iea2=prefsplit(getPref("excludepatterns3").toUpperCase());
		iea2=iea2.map(strToReg);*/
		
		includeexcludearray=includeexcludearray.map(strToReg);
		aewindow.aedump("//includeexcludearray: "+includeexcludearray+"\n",3);
		/*aewindow.aedump("//includeexcludearray: "+iea2+"\n",3);*/		
	}
	//if (getPref("includeenabled")==1) return includeexcludearray.some(new Function("return arguments[0].test('"+filename+"');"));
	//else return includeexcludearray.every(new Function("return !arguments[0].test('"+filename+"');"));
	if (getPref("includeenabled")==1) {
		var test=function(element) {return element.test(filename);};
    	return includeexcludearray.some(test,this);
	}
	else {
		var negtest=function(element) {return !element.test(filename);};
		return includeexcludearray.every(negtest,this);
	}
  }; 
  
  var getPref = (aewindow)? aewindow.attachmentextractor.getPref : new Function();
  var setPref = (aewindow)? aewindow.attachmentextractor.setPref : new Function();
  
  function toString() {
	  return "savefolder: "+(savefolder?savefolder.path:null)+
	  		", selectedURIs: "+selectedURIs+
			", filenamepattern: "+filenamepattern+
			", isBackground: "+isBackground+
			", justDeleteAttachments: "+justDeleteAttachments;
  }; this.toString=toString;
}

function createAEIndTask(savefolder,message,attachments,filenamepattern) {
	aewindow.progress_tracker=progress_tracker;
    aewindow.attachmentextractor=attachmentextractor;
    aewindow.preferenceService=attachmentextractor.preferenceService;
    aewindow.aedump=aedump;
    aewindow.AEMessage=AEMessage;
    aewindow.messenger = messenger;
	aewindow.msgWindow = msgWindow;
	if (!savefolder || !message) {
		aewindow.aedump("// ae aborted because either no message selected or no save path.\n",0);
		return;
	}
	aewindow.currentTask=new AEIndTask(savefolder,message,attachments,filenamepattern,aewindow);
	aewindow.progress_tracker.statusFeedback=aewindow.msgWindow.statusFeedback;
	if (aewindow.currentTask.doDetachConfirm() && !aewindow.attachmentextractor.promptService.confirm(
			null,
			aewindow.attachmentextractor.aeStringBundle.GetStringFromName("ConfirmDetachDialogTitle"),
			aewindow.attachmentextractor.aeStringBundle.GetStringFromName("ConfirmDetachDialogMessage"))) {
				aewindow.aedump("// ae aborted detach confirmation cancelled.\n",1);
				return;
	}	
	aewindow.aedump(aewindow.currentTask+"\n",2);
	aewindow.currentTask.start();
}; 

/* ****************************** AEIndTask ******************************************************** */

function AEIndTask(savefolder,messageUri,attachments,filenamepattern,aewindow) {
  //public vars
  /* this.membername=value */
  if (!filenamepattern) filenamepattern=aewindow.attachmentextractor.getPref("filenamepattern");
  this.filemaker=new AttachmentFileMaker(filenamepattern,savefolder,aewindow);
  this.currentMessage=null;
  this.detachCancellationTimeout=null;
  this.currentDisplayedURI=messageUri;
  this.listeningforMessageId="";
  
  //private vars
  /* var membername=value */
  var that=this;
  var Cc=Components.classes;
  var Ci=Components.interfaces;
  var Cr=Components.results;
  var downloadwindowpref=aewindow.preferenceService.getIntPref("browser.downloadmanager.behavior");
  var downloadwindowdelay=aewindow.preferenceService.getIntPref("browser.download.manager.openDelay");
  var compactfolderspref=aewindow.preferenceService.getBoolPref("mail.prompt_purge_threshhold");
  var includeexcludearray=null;
  var active=false;
  
  //protected methods
  this.start = function() {
	aewindow.aedump('{function:AETask.start}\n',2);
	aewindow.progress_tracker.starting_extraction();
	
	aewindow.preferenceService.setIntPref("browser.downloadmanager.behavior",2);
	aewindow.preferenceService.setIntPref("browser.download.manager.openDelay",30000);
	aewindow.preferenceService.setBoolPref("mail.prompt_purge_threshhold",false);
	//
	that.active=true; 
	aewindow.progress_tracker.reset_tracker();
	that.currentMessage=new aewindow.AEMessage(messageUri,0,aewindow);
	for (var i=0; i<attachments.length;i++) {
		var a=attachments[i];
		if (!a.isExternalAttachment) that.handleAttachment(a.contentType,a.url,a.displayName,a.uri,a.isExternalAttachment);
	}
	aewindow.progress_tracker.starting_message(0,1);	
	that.currentMessage.attachmentextraction(1)
  };
  
  this.selectNextMessage = function() {
    that.endAttachmentextraction();
  };
    
  this.endAttachmentextraction = function() {
	aewindow.aedump('{function:AETask.endAttachmentextraction}\n',2);
	if (!that.active) return;
	that.active=false;
	that.tidyUp();
  };
  
  this.tidyUp = function() {  
	aewindow.aedump('{function:AETask.tidyUp}\n',2);
	aewindow.preferenceService.setIntPref("browser.downloadmanager.behavior",downloadwindowpref);
	aewindow.preferenceService.setIntPref("browser.download.manager.openDelay",downloadwindowdelay);
	aewindow.preferenceService.setBoolPref("mail.prompt_purge_threshhold",compactfolderspref);
	
	aewindow.progress_tracker.ended_extraction();
  }
  
  this.handleAttachment= function(contentType, url, displayName, uri, isExternalAttachment) 
  {
	aewindow.aedump('{function:AETask.handleAttachment}\n',2);  
	if (aewindow.progress_tracker.attachment_busy()) return;
	if (!include_exclude_check2(displayName)) {
  		aewindow.aedump(displayName+" failed include/exclude filename check\n",1);
		return;
  	}
  	if (!mime_check(contentType)) {
		aewindow.aedump(displayName+" failed contentType check\n",1);
		return;
	}
	that.currentMessage.addAttachment(contentType,url,displayName.replace(/ +/g, " "),uri,isExternalAttachment); 
  };
  
  this.isDeleteEnabled=function() {return false;};
  this.isCleartagEnabled=function() {return false;};
  this.isMarkreadEnabled=function() {return false;};
  this.isNotifywhendoneEnabled=function () {return false;};
  this.isSaveMessageEnabled=function() {return false;};
  
  this.isExtractEnabled=function() {
	  return getPref("extract.mode")!=-1;
  }
  
  this.isDetachEnabled=function() {
	  return getPref("actionafterextract.detach");
  };
  
  this.isLaunchEnabled=function() {
	  return getPref("actionafterextract.launch");
  };
  
  this.isEndLaunchEnabled=function() {
	  return getPref("actionafterextract.endlaunch");
  };
  
  this.getOverwritePolicy=function () {
	  return getPref("overwritepolicy");
  };
  
  this.getDetachMode=function() {
	  return getPref("actionafterextract.detach.mode");
  };
  
  this.doDetachConfirm=function() {
	  return that.isDetachEnabled()&& (that.getDetachMode()!=0) && getPref("actionafterextract.detach.confirm");
  };
  
  this.nextUri=function() {
	  return null;
  };
  
  //private methods
  
  function mime_check(mimetype) {
	return (mimetype!="text/x-moz-deleted");
  };
  
  function strToReg(value,index,array) {
	  if (value.charAt(0)=='/') {
		  var expr=value.substring(1,value.lastIndexOf('/'));
		  var flags=value.substring(value.lastIndexOf('/')+1);
		  aewindow.aedump("//directly specificed reg-ex: /"+expr+"/"+flags+"\n",3);
		  return new RegExp(expr,flags);
	  }
	  value=value.replace(".","\\.");
	  value=value.replace("*",".*");
	  value="^"+value+"$";
	  return new RegExp(value,"i");
  }
  
  function include_exclude_check2(filename) {
	filename=filename.replace(/'/g,"\'");
	if (includeexcludearray==null) {
		if (getPref("includeenabled")==1) includeexcludearray=getPref("includepatterns4").split(';');
		else includeexcludearray=getPref("excludepatterns4").split(';');
		
		/*var iea2=prefsplit(getPref("excludepatterns3").toUpperCase());
		iea2=iea2.map(strToReg);*/
		
		includeexcludearray=includeexcludearray.map(strToReg);
		aewindow.aedump("//includeexcludearray: "+includeexcludearray+"\n",3);
		/*aewindow.aedump("//includeexcludearray: "+iea2+"\n",3);*/		
	}
	//if (getPref("includeenabled")==1) return includeexcludearray.some(new Function("return arguments[0].test('"+filename+"');"));
	//else return includeexcludearray.every(new Function("return !arguments[0].test('"+filename+"');"));
	if (getPref("includeenabled")==1) {
		var test=function(element) {return element.test(filename);};
    	return includeexcludearray.some(test,this);
	}
	else {
		var negtest=function(element) {return !element.test(filename);};
		return includeexcludearray.every(negtest,this);
	}
  }; 
  
  var getPref = (aewindow)? aewindow.attachmentextractor.getPref : new Function();
  var setPref = (aewindow)? aewindow.attachmentextractor.setPref : new Function();
  
  function toString() {
	  return "savefolder: "+(savefolder?savefolder.path:null)+
	  		", messageUri: "+messageUri+
			", filenamepattern: "+filenamepattern;
  }; this.toString=toString;
}

/* ******************************** AEMessage ******************************************************* */

function AEMessage(uri,messageIndex,pobj) 
{
	var that=this;
	var Cc=Components.classes;
    var Ci=Components.interfaces;
    var Cr=Components.results;
	
	var	attachments_ct=new Array();
	var	attachments_url=new Array();
	var	attachments_display=new Array();
	var	attachments_uri=new Array();
	var attachments_appendage=new Array();
	var attachments_savedfile=new Array();
	var aewindow=pobj;
    var deleted=false;
	var headerDataCache=new AttachmentFileMakerCache();
	var detachTempFile=null;
	var zerofileTimeout=null;
	var cleanUpFilterTimeout=null;
	var started=false;
	var protocol=uri.substr(0,uri.indexOf(":"));
	
	var protocols={NEWS:"news",IMAP:"imap",POP:"mailbox-message"};
	
  //protected methods
  this.attachmentextraction = function(numURIs) {
	started=true;
	if (aewindow.currentTask.isExtractEnabled()&&getNumAttachments()>0) this.saveAtt(0);	
	else {
		aewindow.aedump("// no attachments to extract in this message\n",3);
		this.doMarkread();
	}
  };
  
  this.saveAtt= function(attachmentindex) {
	aewindow.aedump('{function:AEMessage.saveAtt('+attachmentindex+')}\n',2);
  	aewindow.progress_tracker.starting_attachment(attachmentindex,getNumAttachments());
    	 
  	var attachment = getAttachment(attachmentindex);
  	var file=aewindow.currentTask.filemaker.make(attachment.displayName,headerDataCache);
    if (file&&file.parent&&!file.parent.exists()) file.parent.create(file.DIRECTORY_TYPE,0600);
  	//aewindow.aedump(attachment.uri+"\n"+attachment.url+"\n"+attachment);
  
  	if (file) {
      attachments_appendage[attachmentindex]=aewindow.currentTask.filemaker.lastMadeAppendage;
      try {
		if (getPref("extract.mode")==0) {
		  attachments_savedfile[attachmentindex]=aewindow.messenger.saveAttachmentToFolder(
	 		attachment.contentType,
			attachment.url,
			encodeURIComponent(file.leafName),
			attachment.uri,
			file.parent);
		}else{
		  attachments_savedfile[attachmentindex]=aeMessenger.saveAttachmentToFolder(
	 		attachment.contentType,
			attachment.url,
			file.leafName,
			attachment.uri,
			file.parent,
			attachmentindex);
		}
		file=attachments_savedfile[attachmentindex];
      } catch (e) {
		  aewindow.aedump(e+"\n");
		  file=null;
	  }
	} 
	if (!file) {
		attachments_ct[attachmentindex]=undefined;
		attachments_url[attachmentindex]=undefined;
		attachments_display[attachmentindex]=undefined;
		attachments_uri[attachmentindex]=undefined;
		attachments_savedfile[attachmentindex]=undefined;
		attachments_appendage[attachmentindex]=undefined;
	}
  	if (!file||getPref("extract.mode")==0) {
		aewindow.currentTask.currentMessage.saveAtt_cleanUpFilter(attachmentindex);
		zerofileTimeout=setTimeout(aewindow.currentTask.currentMessage.saveAtt_cleanUp,5000,attachmentindex,false);
	}
  };

  this.saveAtt_cleanUpFilter=function(attachmentindex) {
	var sfile=attachments_savedfile[attachmentindex];
	if (sfile==null || (sfile.exists() && sfile.fileSize>0 /*aewindow.downloadManager.activeDownloadCount==0*/)  ) {
		aewindow.currentTask.currentMessage.saveAtt_cleanUp(attachmentindex,false);
	}
	else cleanUpFilterTimeout=setTimeout(aewindow.currentTask.currentMessage.saveAtt_cleanUpFilter,5,attachmentindex);
  };

  this.saveAtt_cleanUp= function(attachmentindex,failure) {
	aewindow.aedump('{function:AEMessage.saveAtt_cleanUp('+attachmentindex+','+failure+')}\n',2);
	clearTimeout(zerofileTimeout);
	if (failure) {
		aedump("//ae: delayed saving failure notification.\n",1);
		attachments_ct[attachmentindex]=undefined;
		attachments_url[attachmentindex]=undefined;
		attachments_display[attachmentindex]=undefined;
		attachments_uri[attachmentindex]=undefined;
		attachments_savedfile[attachmentindex]=undefined;
	}
	try{ 
	  if (aewindow.currentTask.isLaunchEnabled()&&
			attachments_savedfile[attachmentindex]!=null && attachments_savedfile[attachmentindex].exists()) {
		aewindow.aedump('{function:AEMessage.doLaunch}\n',2);
		attachments_savedfile[attachmentindex].launch();
	  } 
	} catch (e) {aewindow.aedump(e+"\n");}
	aewindow.progress_tracker.stopping_attachment(attachmentindex);
	attachmentindex++;
  	if (attachmentindex>=getNumAttachments()) {
		setTimeout(that.doMarkread,getPref("nextattachmentdelay"));
	}
  	else {
		setTimeout(aewindow.currentTask.currentMessage.saveAtt,getPref("nextattachmentdelay"),attachmentindex);
	}
  };
  
  this.doMarkread= function() {
	/*if (aewindow.downloadManager.activeDownloadCount!=0) {
		setTimeout(that.doMarkread,10);
		return;
	}*/
	if (aewindow.currentTask.isMarkreadEnabled()) {
		aewindow.aedump('{function:AEMessage.doMarkread}\n',2);
		aewindow.progress_tracker.starting_markread();
		aewindow.gDBView.doCommand(aewindow.nsMsgViewCommandType.markMessagesRead);
	}
	if (aewindow.currentTask.isSaveMessageEnabled()) { 
	  aewindow.aedump('{function:AEMessage.doSaveMessage}\n',2);
	  aeMessenger.saveMessageToDisk(aewindow.currentTask.currentDisplayedURI,
									aewindow.currentTask.filemaker.makeSaveMessage(headerDataCache));
	  return;
	}
	that.doCleartag();
  }
  
  this.doCleartag= function() {
	if (aewindow.currentTask.isCleartagEnabled()&&!isNewsMessage()) {
		aewindow.aedump('{function:AEMessage.doCleartag}\n',2);
		var uriarray = aewindow.newnsIArray();
		var triggerTag=getPref("autoextract.triggertag");
		var hdr=that.getMessageHeader();
		uriarray.AppendElement(hdr);
		try { hdr.folder.removeKeywordFromMessages(uriarray,triggerTag);
		}catch (e) {aewindow.aedump("// removeKeywordFromMessages throws error: "+e+"\n",1);}
	}
  
	if (aewindow.currentTask.isDetachEnabled()&&!isNewsMessage()&&!isRSSMessage()) {
		aewindow.aedump('{function:AEMessage.doDetach}\n',2);
		aewindow.progress_tracker.starting_detach();
		aewindow.currentTask.listeningforMessageId=that.getMessageHeader().messageId;
		var acl=arraycompact(attachments_ct);
		if (acl.length>0) {
			if (aewindow.currentTask.getDetachMode()!=0) { 
				var deleteAtt=(aewindow.currentTask.getDetachMode()==1) || !aewindow.currentTask.isExtractEnabled();
				var savedfiles=(deleteAtt)? null : arraycompact(attachments_savedfile);
				detachTempFile=aeMessenger.detachAttachments(aewindow.messenger,aewindow.msgWindow,acl,
															 arraycompact(attachments_url),
															 arraycompact(attachments_display),
															 arraycompact(attachments_uri),
															 savedfiles);
			} else {
				aewindow.messenger.detachAllAttachments(acl.length,acl,
															 arraycompact(attachments_url),
															 arraycompact(attachments_display), 
															 arraycompact(attachments_uri),
															 false);
			}				
			aewindow.currentTask.detachCancellationTimeout=setTimeout(aewindow.currentTask.currentMessage.doDelete,5000);
		return;
	}
	}
	that.doDelete();
  };
  
  this.doDelete= function() {
	//aewindow.aedump('//doDelete\n',0);
	aewindow.currentTask.listeningforMessageId="";
	if (deleted) return;
	else deleted=true;
	if (detachTempFile) {
      aewindow.aedump('{function:AEMessage.deleteDetachTempfile}\n',2);
      try {
          detachTempFile.remove(false);
      } catch (e) {aewindow.aedump(e);}
    }
	aewindow.progress_tracker.stopping_message(messageIndex);
	if (aewindow.currentTask.isDeleteEnabled()&&!isNewsMessage()) {
		aewindow.aedump('{function:AEMessage.doDelete}\n',2);
		/*aewindow.gDBView.doCommand(aewindow.nsMsgViewCommandType.deleteMsg);
	 	aewindow.gDBView.onDeleteCompleted(true);*/
		var messageArray = aewindow.newnsIArray();
  		var hdr=that.getMessageHeader();
		messageArray.AppendElement(hdr);
		function DCopyListener(aewindow,delay) { 
		  this.OnStartCopy=function() {}
		  this.OnProgress=function(aProgress,aProgressMax) {}
		  this.GetMessageId=function() {return null}
		  this.SetMessageKey=function(aKey) {}
		  this.OnStopCopy=function(aStatus) {setTimeout(aewindow.currentTask.selectNextMessage,delay);}		
		};
  		var dCopyListener=new DCopyListener(aewindow,getPref("nextmessagedelay"));
		hdr.folder.deleteMessages(messageArray,aewindow.msgWindow,false,false,dCopyListener,true); 
		return;
	}
	setTimeout(aewindow.currentTask.selectNextMessage,getPref("nextmessagedelay"));
  };
  
  function getMessageHeader() {
	  var uri=aewindow.currentTask.currentDisplayedURI;
	  //aewindow.aedump(uri+" : "+aewindow.messenger.messageServiceFromURI(uri).messageURIToMsgHdr(uri)+"\n",3);
	  return aewindow.messenger.messageServiceFromURI(uri).messageURIToMsgHdr(uri);
  }
  this.getMessageHeader=getMessageHeader;
  
  this.getSavedFile=function(attachmentindex) {
	return attachments_savedfile[attachmentindex];
  };
  
  this.getAppendage=function(attachmentindex) {
	return attachments_appendage[attachmentindex];
  };
  
  function postProcessMessage(html) {
	/*aedump("[["+html+"]]\n"); */
	//aedump(attachments_url+"\n");
	//aedump(attachments_display+"\n");
	
	function replacer(str,p1,p2,p3) {
		//aedump(str+", ");
		var dn=/&filename=([^&]*)/.exec(p2);
		dn=decodeURIComponent(dn[1]);
		//aedump(dn+"\n");
		for (var i=0;i<attachments_display.length;i++) {
			if (attachments_display[i]==dn) {
			  if (attachments_savedfile[i]) return p1+encodeURIComponent(attachments_appendage[i])+p3;
			  else break;
			}
		}	  
		return str;
	}
	return html.replace(/(<IMG SRC=")(?:mailbox|imap|news)-message([^"]*)(">)/gi,replacer);
  }
  this.postProcessMessage=postProcessMessage;
  
  //private methods
  
  function getNumAttachments() {
	  return attachments_ct.length;
  };
  this.getNumAttachments=getNumAttachments; 
  
  function addAttachment(contentType, url, displayName, uri, isExternalAttachment) {
	if (started) {
		aewindow.aedump("// AE: Extraction already started but TB adding more attachments.  Ignoring. \n",1);
		return;
	}
	//aedump("{function:AEMessage.addAttachment("+contentType+","+url+","+displayName+","+uri+","+isExternalAttachment+")\n",3);
	attachments_ct.push(contentType);
	attachments_url.push(url);
	attachments_display.push(displayName);
	attachments_uri.push(uri);
  };
  this.addAttachment=addAttachment;
  
  function getAttachment(index) {
	return new CreateNewAttachmentInfo(attachments_ct[index],attachments_url[index],attachments_display[index],
										attachments_uri[index],false);
  };
  this.getAttachment=getAttachment;
  
  function CreateNewAttachmentInfo(contentType, url, displayName, uri, isExternalAttachment)
  {
	  this.contentType = contentType;
	  this.url = url;
	  this.displayName = displayName;
 	  this.uri = uri;
 	  this.isExternalAttachment = isExternalAttachment;
  };
  
  function arraycompact(array) {
	var out=new Array();
	for (var i=0;i<array.length;i++) {if (!(array[i]===undefined)) out.push(array[i]);}
	return out;
  };
  
  function arraycopy(ain) {
	var aout= new Array(ain.length);
	for (var i=0;i<aout.length;i++) {aout[i]=ain[i];}
	return aout;
  };
  
  function isNewsMessage() {
	  return (protocol==protocols.NEWS);
  }
  
  function isRSSMessage() {
	  return (protocol==protocols.RSS);
  }
  
  //   
  var getPref = (aewindow)? aewindow.attachmentextractor.getPref : new Function();
  var setPref = (aewindow)? aewindow.attachmentextractor.setPref : new Function();
};

function AttachmentFileMakerCache() {
	this.datetime=null;
	this.author=null;
	this.authoremail=null;
	this.subject=null;
	var that=this;
	this.toString=function() {
		return "["+that.datetime+","+that.author+","+that.authoremail+","+that.subject+"]";
	}
};

function AttachmentFileMaker(currentfilenamepattern,folder,aewindow) {
  var that=this;

  if ((typeof folder)=="string"|| folder instanceof String) {
		this.destFolder=aewindow.makeFile(); 
		this.destFolder.initWithPath(folder);
  }else this.destFolder=folder;
  
  this.lastMadeAppendage=null;
  
  function make(filename,metacache) {	
	if (!filename) return false;
	var folder=that.destFolder;
	
	var proposedfileobject = aewindow.makeFile();
	var count_pattern=getPref("filenamepattern.countpattern");
	var policy=aewindow.currentTask.getOverwritePolicy();
	
	proposedfileobject.initWithFile(folder);
	//aewindow.aedump("cfnp: "+this.currentfilenamepattern);
	
	var proposedappendage = substitutetokens(currentfilenamepattern.replace(/#count#/g,""),folder,filename,0,metacache);
	proposedfileobject.appendRelativePath(proposedappendage);
	
	if (policy == aewindow.attachmentextractor.OVERWRITEPOLICY_REPLACE) {
		aewindow.aedump(">> '"+proposedappendage+"' - replace so no need to check.  proceed\n",1);
		that.lastMadeAppendage=proposedappendage;
		return proposedfileobject;
	}
	
	if (!proposedfileobject.exists()) {
		aewindow.aedump(">> '"+proposedappendage+"' doesn't exist so proceed\n",1);
		that.lastMadeAppendage=proposedappendage;
		return proposedfileobject;
	}
	else if (policy == aewindow.attachmentextractor.OVERWRITEPOLICY_IGNORE) {
		aewindow.aedump(">> '"+proposedappendage+"' exists and policy is to skip so abort extraction\n",1);
		return false;
	}
	else if (policy == aewindow.attachmentextractor.OVERWRITEPOLICY_ASK) {
		var extbundle = aewindow.attachmentextractor.aeStringBundle;
			
		var promptService = aewindow.attachmentextractor.promptService;
		var flags=promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_0 +
	          promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_1 +
    	      promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_2;

		var dmessage= extbundle.GetStringFromName("OverwriteDialogMessage").replace(/%/g,proposedappendage);
		var checkedstate={};
		var selectedbutton= promptService.confirmEx(window,
								extbundle.GetStringFromName("OverwriteDialogTitle"),
								dmessage,
								flags, 
								extbundle.GetStringFromName("OverwriteDialogReplace"), 
								extbundle.GetStringFromName("OverwriteDialogRename"), 
								extbundle.GetStringFromName("OverwriteDialogIgnore"), 
								extbundle.GetStringFromName("OverwriteDialogDontAskAgain"), 
								checkedstate);
		if (checkedstate.value==true) {
			setPref("overwritepolicy",selectedbutton+1);
		}
		if (selectedbutton==0) { 
			that.lastMadeAppendage=proposedappendage;
			return proposedfileobject;
		}
		if (selectedbutton==1) { 
			return iterativegenerator(folder, filename ,proposedfileobject,currentfilenamepattern.replace(/#count#/g,count_pattern),1,metacache);					
		}
		if (selectedbutton==2) { 
			return false;
		}
		return false;
	}
	else return iterativegenerator(folder, filename ,proposedfileobject,currentfilenamepattern.replace(/#count#/g,count_pattern),1,metacache);	
  };
  this.make = make;  
  
  function makeSaveMessage(metacache) {
	var proposedfileobject = aewindow.makeFile();
	var messagetext_pattern=getPref("filenamepattern.savemessage");
	var count_pattern=getPref("filenamepattern.savemessage.countpattern");
	
	proposedfileobject.initWithFile(folder);
	
	var proposedappendage = substitutetokens(messagetext_pattern.replace(/#count#/g,""),folder,"",0,metacache);
	proposedfileobject.appendRelativePath(proposedappendage);
	
	if (!proposedfileobject.exists()) {
		that.lastMadeAppendage=proposedappendage;
		return proposedfileobject;
	}
	else return iterativegenerator(folder,"",proposedfileobject,messagetext_pattern.replace(/#count#/g,count_pattern),1,metacache);
  };
  this.makeSaveMessage=makeSaveMessage;
  
  //   
  function formatdatestring(format,date) {
	var out="";
	for (var i=0;i<format.length;i++) {
		switch (format.charAt(i)) {
			case "d": out+=padZeros(date.getDate(),2); break;
			case "D": out+=SHORTDAYS[date.getDay()]; break;
			case "j": out+=date.getDate(); break;
			case "l": out+=FULLDAYS[date.getDay()]; break;
			case "S": out+=DAYSUFFIX[date.getDate()]; break;
			case "F": out+=FULLMONTHS[date.getMonth()]; break;
			case "m": out+=padZeros(date.getMonth()+1,2); break;
			case "M": out+=SHORTMONTHS[date.getMonth()]; break;
			case "n": out+=date.getMonth()+1; break;
			case "Y": out+=date.getFullYear(); break;
			case "G": out+=date.getHours(); break;
			case "H": out+=padZeros(date.getHours(),2); break;
			case "i": out+=padZeros(date.getMinutes(),2); break;
			case "s": out+=padZeros(date.getSeconds(),2); break;
			default : out+=format.charAt(i);
		}
	}
	return out;
  };
  this.formatdatestring=formatdatestring;
  
  //   
  function iterativegenerator(folder,file,proposedfileobject,filename_pattern,count,cache)
  {
	var proposedappendage;
	do {
		proposedfileobject.initWithFile(folder);
		proposedappendage=substitutetokens(filename_pattern,folder,file,count,cache);
		proposedfileobject.appendRelativePath(proposedappendage);
		count++;
	} while (proposedfileobject.exists())
	if (file!="") aewindow.aedump(">> '"+proposedappendage+"' doesn't exist so proceed\n",1);
	that.lastMadeAppendage=proposedappendage;
	return proposedfileobject;
  };
  this.iterativegenerator=iterativegenerator;

  function substitutetokens_sub(stringin,folder,filename,count,subject,date,author,authoremail)
  {
	var extpart="";
        if ((stringin.indexOf("#namepart#")!= -1) || (stringin.indexOf("#extpart#")!= -1)) {
		var namepart="";
		if (filename.lastIndexOf(".")!= -1) {
			namepart=filename.substring(0,filename.lastIndexOf("."));
			extpart=filename.substring(filename.lastIndexOf("."));
		}
		else namepart=filename;
		stringin=stringin.replace(/#namepart#/g,namepart);
	}
	return validateFileName(folder,stringin.replace(/#date#/g,date).replace(/#from#/g,author).replace(/#fromemail#/g,authoremail).replace(/#subject#/g,subject),extpart,count);
  };
  this.generate=substitutetokens_sub;

  function substitutetokens(stringin, folder,filename,count,cache)
  {
	if (!cache) cache=new AttachmentFileMakerCache();
	var msheader= aewindow.currentTask.currentMessage.getMessageHeader(); 
	if (!cache.datetime&&(stringin.indexOf('#date#')>-1)) {
			cache.datetime=formatdatestring(getPref("filenamepattern.datepattern"),new Date(msheader.dateInSeconds*1000)); 	
	} 
	if (!cache.author&&(stringin.indexOf('#from#')>-1)) {
		cache.author=(msheader.mime2DecodedAuthor?msheader.mime2DecodedAuthor+"":"").replace(/ <.*>|[\/\\#]+/g, "");
	}
	if (!cache.authoremail&&(stringin.indexOf('#fromemail#')>-1)) {
		cache.authoremail=(msheader.mime2DecodedAuthor?msheader.mime2DecodedAuthor+"":"").replace(/.*<|>.*|[\/\\#]+/g, "");
	}
	/*if (!cache.authoremail&&(stringin.indexOf('#toemail#')>-1)) {
		//cache.authoremail=(msheader.mime2DecodedAuthor?msheader.mime2DecodedAuthor+"":"").replace(/.*<|>.*|[\/\\#]+/g, "");
	}*/
	if (!cache.subject&&(stringin.indexOf('#subject#')>-1)) {
		cache.subject=(msheader.mime2DecodedSubject?msheader.mime2DecodedSubject+"":"").replace(/[\/\\#]+/g, "_");
		if (getPref("filenamepattern.cleansubject")) {
			var starts=getPref("filenamepattern.cleansubject.strings").toLowerCase().split(',');
			cache.subject=cleanSubjectLine(cache.subject,starts);
		}
	}
	aewindow.aedump("cache: "+cache+"\n",3);
	//aewindow.aedump("m2da: "+msheader.mime2DecodedAuthor+", a: "+msheader.author+"\n",0);
	return substitutetokens_sub(stringin, folder,filename, count, cache.subject ,cache.datetime, cache.author, cache.authoremail);
  };
  
  //   
  function cleanSubjectLine(subject,starts) {
	  var morework;
	  if (subject==null) return "";
	  do {
	  	morework=false;
	  	if (subject.length==0) break;
		if (subject.charAt(0)=='[' && subject.charAt(subject.length-1)==']') {
			subject=subject.substring(1,subject.length-1);
			morework=true;
		}
	  	for (var i = 0; i < starts.length && !morework; i++) {
	  		if (subject.toLowerCase().indexOf(starts[i])==0) {
				subject=subject.substring(starts[i].length);
				morework=true;
			}
	  	}
	  } while (morework);
	  return subject;
  };
  this.cleanSubjectLine=cleanSubjectLine;

  function padZeros(num, length) {
    num=num.toString();             
    var zeros=length-num.length; 
    for (var i=0; i<zeros; i++) {
      num="0"+num;
    }
    return num;
  };

  const DAYSUFFIX= Array("",
						 "st","nd","rd","th","th","th","th","th","th","th",
						 "th","th","th","th","th","th","th","th","th","th",
						 "st","nd","rd","th","th","th","th","th","th","th",
						 "st");
  const SHORTDAYS= Array("Sun","Mon","Tue","Wed","Thu","Fri","Sat");
  const FULLDAYS= Array("Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday");
  const SHORTMONTHS=Array("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");
  const FULLMONTHS=Array("January","Febuary","March","April","May","June","July","August","September","October","November","December");
  
  //   
  function validateFilenamePattern(fnp) {
	  if (!fnp) return fnp;
	  if (fnp.indexOf("%")!=-1 || fnp.indexOf("#count#")!=-1) return fnp;
	  var expi=fnp.indexOf("#extpart#");
	  if (expi!=-1) return fnp.substring(0,expi)+"#count#"+fnp.substring(expi,fnp.length);
	  else return fnp+"#count#";
  };
  this.validateFilenamePattern=validateFilenamePattern;
		  
  function validateCountPattern(cp) {
	  return (cp.indexOf("%")!=-1)? cp : cp+"%";
  };
  this.validateCountPattern=validateCountPattern;
  
  /* *** contribution below from Matteo F Zazzetta.  Support functions to rewritten validateFilename() *** */

  function countOccurrences(string,substring,rep) {
      var idx = 0;
      var arr = new Array();
      while ((idx = string.indexOf(substring,idx))>=0) {
        arr.push(new StrIdx(idx++,substring,rep));
      }
      return arr;
    };
    
    function joinArrs(a1,a2) {
      var ra = new Array()
      for (var i=0; i<a1.length; i++) {
        ra.push(a1[i]);
      }
      for (var i=0; i<a2.length; i++) {
        ra.push(a2[i]);
      }
      ra.sort(sortStrIdx);
      return ra;
    }
    
    function StrIdx(i,s,r) {
      this.idx = i;
      this.string = s; 
      this.replace = r;
      this.toString = function() {
        return ""+this.idx+" "+this.string+" "+this.replace;
      };
    };
    
    function sortStrIdx(a,b) {
      return a.idx-b.idx;
    };
    
    function reconstructString(string,idxarr) {
      var rs = string;
      var offset = 0;
      for (var i=0; i<idxarr.length; i++) {
        var pos = offset+idxarr[i].idx;
        if (pos<rs.length) { 
          var newrs = "";
          if (pos>0) {
            newrs = rs.substring(0,pos);
          }
          rs = newrs+idxarr[i].replace+rs.substring(pos);
        }
        else {
          rs+=idxarr[i].replace;
        }
        offset += (idxarr[i].replace.length-idxarr[i].string.length);
      }
      return rs;
    };
    
 /* function below based on seamonkey/toolkit/content/contentAreaUtils.js but modifed to take into not filter out folder seperators*/

    function validateFileName(folder,aFileName,extpart,count) {
	  count = ""+count;    
   if (navigator.appVersion.indexOf("Windows") != -1) {
     aFileName = aFileName.replace(/[\"]+/g, "'");			/*  " = '			*/
     aFileName = aFileName.replace(/[\*\:\?]+/g, " ");  	/*  * : ? = 		*/
     aFileName = aFileName.replace(/[\<]+/g, "(");			/*  < = (			*/
     aFileName = aFileName.replace(/[\>]+/g, ")");			/*  > = )			*/
     aFileName = aFileName.replace(/[\/\|]+/g, "_");        /*  /| = _ 		    */
     var comp = (folder==null||folder.path==null?"":folder.path) + "\\" + aFileName;
            comp = comp.replace(/\\/g,"");
            var len1 = comp.length;
     	var len2 = comp.replace(/%/g,count).replace(/#extpart#/g,extpart).length;
            var len3 = comp.replace(/%/g,"").replace(/#extpart#/g,"").length;
            if (len2 >= 250) {
                var splits = aFileName.split("\\");
                var cd = 0;
                var totlen = aFileName.length-2*splits.length+1+len3-len1;
                var charsToDelete = len2-250;
                var remainToDelete = charsToDelete;
                if (charsToDelete<=totlen) {
                    var newfn = "";
                    for (var i=0; i<splits.length; i++) {
                        var arr = countOccurrences(splits[i],"%",count);
                        arr = joinArrs(arr,countOccurrences(splits[i],"#extpart#",extpart));
                        //arr = joinArrays(arr1,arr2);
                        splits[i] = splits[i].replace(/%/g,"").replace(/#extpart#/g,"");
                        cd = (i==splits.length-1)?remainToDelete:Math.min(Math.ceil((splits[i].length-1)/totlen*charsToDelete),splits[i].length-1);
                        remainToDelete-=cd;
                        splits[i] = splits[i].substring(0,splits[i].length-cd);
                        splits[i] = reconstructString(splits[i],arr);
                        newfn+=(i==0?"":"\\")+splits[i];
                    }
                    aFileName = newfn;
                }
            }
        else aFileName = aFileName.replace(/%/g,count).replace(/#extpart#/g,extpart);
            aFileName = aFileName.replace(/[\. ]+\\/g,"\\");    
   } 
   else if (navigator.appVersion.indexOf("Macintosh") != -1) {
     aFileName = aFileName.replace(/[\:]+/g, "_");		/*   : = _ 	*/
        aFileName = aFileName.replace(/%/g,count).replace(/#extpart#/g,extpart);
   }
   else {
       aFileName = aFileName.replace(/%/g,count).replace(/#extpart#/g,extpart);
   }
   return aFileName;
  }
  
  /* ** contribution from Matteo F Zazzetta end ** */
  
  var getPref = (aewindow)? aewindow.attachmentextractor.getPref : new Function();
  var setPref = (aewindow)? aewindow.attachmentextractor.setPref : new Function();
};

function aeMsgDBViewCommandUpdater() {}
aeMsgDBViewCommandUpdater.prototype = 
{
  updateCommandStatus : function() {},
  displayMessageChanged : function(aFolder, aSubject, aKeywords) {},
  updateNextMessageAfterDelete : function() {},
  QueryInterface : function(iid) {
    if (iid.equals(Components.interfaces.nsIMsgDBViewCommandUpdater) ||
        iid.equals(Components.interfaces.nsISupports)) return this;
    throw Components.results.NS_NOINTERFACE;
  }
}

// 
function AEMsgWindowCommands(task) {
  this.SelectFolder= function(folderUri) {return task.SelectFolder(folderUri);};
  this.SelectMessage= function(messageUri) {return task.SelectMessage(messageUri);};
};
