/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Attachment Extractor.
 *
 * The Initial Developer of the Original Code is
 * Andrew Williamson <eviljeff@eviljeff.com>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function AttachmentExtractor() {
/* constants */
this.OVERWRITEPOLICY_ASK=0;
this.OVERWRITEPOLICY_REPLACE=1;
this.OVERWRITEPOLICY_RENAME=2;
this.OVERWRITEPOLICY_IGNORE=3;
this.MRUMAXCOUNT=20;
var AE_VERSION='1.1.1';
/* variables */
this.autoUris=new Array();
var that=this;
var Cc=Components.classes;
var Ci=Components.interfaces;
var Cr=Components.results;
var toggleMRUdone=false;

/* services */
this.preferenceService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
this.preferenceService2 = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);

this.strBundleService = (Cc["@mozilla.org/intl/stringbundle;1"].getService()).QueryInterface(Ci.nsIStringBundleService);
this.promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);

/* bundles */
this.aeStringBundle=this.strBundleService.createBundle("chrome://attachmentextractor/locale/attachmentextractor.properties");
this.messengerStringBundle=this.strBundleService.createBundle("chrome://messenger/locale/messenger.properties");

this.init = function() {
	createaedumpfile();
	convertPrefs();
	setTimeout(updatecheck,1000);
	
	checkForAutoTag();
	Cc["@mozilla.org/messenger/services/session;1"].getService(Ci.nsIMsgMailSession).AddFolderListener(
			aefolderListener, Ci.nsIFolderListener.added + Ci.nsIFolderListener.propertyChanged /*Ci.nsIFolderListener.all*/);
	addEventListener('load', setupListeners, true);
	addEventListener('unload', unSetupListeners, true);  //unnessecary?
	
	that.preferenceService.QueryInterface(Ci.nsIPrefBranch2).addObserver("attachmentextractor.savepathmru", new AEPrefObserver(), false);
};

/* shortcut functions to get & set ae's preferences. Only for simple type prefs on attachmentextractor branch.*/
this.getPref = function(pref) {
	var ps=that.preferenceService2.getBranch("attachmentextractor.");
	var type= ps.getPrefType(pref);
	if (type==ps.PREF_BOOL) return ps.getBoolPref(pref);
	if (type==ps.PREF_INT) return ps.getIntPref(pref);
	if (type==ps.PREF_STRING) return ps.getCharPref(pref);
	return null;
};
var getPref=this.getPref;

this.setPref = function(pref,value) {
	var ps=that.preferenceService2.getBranch("attachmentextractor.");
	var type= ps.getPrefType(pref);
	if (type==ps.PREF_BOOL) return ps.setBoolPref(pref,value);
	if (type==ps.PREF_INT) return ps.setIntPref(pref,value);
	if (type==ps.PREF_STRING) return ps.setCharPref(pref,value);
	return null;
};
var setPref=this.setPref;

/* access functions */

this.doPatternAttachmentextraction = function(saveselect,all) {
	if (!all&&!GetSelectedMessages()) return;
	var fnp=getFilenamePattern();
	if (!fnp) return;
	this.doAttachmentextraction(saveselect,all,fnp);
};

// 
this.doAttachmentextraction = function(saveselect,all,fnp){
	var folder=null;
	var deep=(all==2);
	saveselect=saveselect+"";
	//aedump("//ae: saveselect: "+saveselect+" all: "+all+"\n");
	switch (saveselect) {
		case "default": folder=getDefaultSaveFolder(); break;
		case "deleteAtt": this.startAttachmentextraction(null,(all)?collectMessagesFromFolder(deep):GetSelectedMessages(),null,false,true); break;
		case "0": folder=getSaveFolder("messenger.save.dir",true); if (folder) addToMRUList(folder); break;
		default : folder=useMRU(saveselect);
	}
	if (folder) this.startAttachmentextraction(folder,(all)?collectMessagesFromFolder(deep):GetSelectedMessages(),fnp,false,false);
};

this.doIndividualAttachmentextraction = function(saveselect,all,fnp){
	var folder;
	saveselect=saveselect+"";
	//aedump("//ae: saveselect: "+saveselect+" \n");
	switch (saveselect) {
		case "default": folder=getDefaultSaveFolder(); break;
		case "0": folder=getSaveFolder("messenger.save.dir",true); if (folder) addToMRUList(folder); break;
		default : folder=useMRU(saveselect);
	} 
	if (folder) this.startIndividualAttachmentextraction(folder,GetLoadedMessage(),(all)?currentAttachments:getSelectedAttachments(),fnp);
};

this.doBackgroundAttachmentextraction = function() {
	this.startAttachmentextraction(getAutoSaveFolder(),that.autoUris,null,true,false);
	that.autoUris=new Array();
};

this.doSingleBackgroundAttachmentextraction = function(uri) {
	this.startAttachmentextraction(getAutoSaveFolder(),[uri],null,true,false);
};


/* begining and ending functions */
this.startAttachmentextraction = function(savefolder,messages,filenamepattern,background,deleteAtt) {
	if (getPref("queuerequests")) {
		// open up existing window and add items to task.
		var aew=progress_tracker.getWindowByType("mail:AEDialog");
		if (aew) {
			//aedump("// ** found aew and continuing ** \n");
			aew.aewindow.queueAETask(savefolder,messages,filenamepattern,gDBView,background,deleteAtt);
			return;
		}
	}
	window.openDialog("chrome://attachmentextractor/content/aewindow.xul",
				"_blank", "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar",
		savefolder,messages,filenamepattern,gDBView,background,deleteAtt);
}; 

this.startIndividualAttachmentextraction = function(savefolder,message,attachments,filenamepattern) {
	createAEIndTask(savefolder,message,attachments,filenamepattern);
	
};

/* anxillary functions */

function collectMessagesFromFolder(deep) {
	var view = GetDBView();
	var treeView = view.QueryInterface(Ci.nsITreeView);
	var count = treeView.rowCount;
	if (!count) return false;
	var mOut=Array(count);
	for (var i = 0; i < count; i++) {
		var msgURI = view.getURIForViewIndex(i);
		mOut[i]=msgURI;
	}
	if (deep && view.msgFolder.hasSubFolders) {
		var enumr=view.msgFolder.GetSubFolders();
		enumr.first();
		try {
		  while (!enumr.isDone()) {
			getAllMessages_sub(enumr.currentItem().QueryInterface(Ci.nsIMsgFolder),mOut,deep);
			if (!enumr.isDone()) enumr.next();
		  }
		} catch (e) {}		
	}
	return mOut;
};

function getAllMessages(deep) {
	var msgs=new Array();
	var msgFolder = GetDBView().msgFolder;
	getAllMessages_sub(msgFolder,msgs,deep);
	return msgs;
};

function getAllMessages_sub(folder,msgs,deep) {
	var enumr=folder.getMessages(msgWindow);
	while (enumr.hasMoreElements()) {
		var msg=enumr.getNext().QueryInterface(Ci.nsIMsgDBHdr);
		if (msg) msgs.push(folder.generateMessageURI(msg.messageKey));
	}
	if (deep && folder.hasSubFolders) {
		enumr=folder.GetSubFolders();
		enumr.first();
		try {
		  while (!enumr.isDone()) {
			getAllMessages_sub(enumr.currentItem().QueryInterface(Ci.nsIMsgFolder),msgs,deep);
			if (!enumr.isDone()) enumr.next();
		  }
		} catch (e) {}		
	}
}

function getSelectedAttachments() {
	var attachmentList = document.getElementById('attachmentList');
	var selectedAttachments = attachmentList.selectedItems;
	var atts=new Array();
	for (var i = 0; i < selectedAttachments.length; i++) {
		atts[i]=selectedAttachments[i].attachment;
		atts[i].uri=atts[i].messageUri;
		/*aAttachment.contentType, 
		aAttachment.url, 
		encodeURIComponent(aAttachment.displayName), 
		aAttachment.messageUri, aAttachment.isExternalAttachment*/
	}
	return atts;
}

function getSaveFolder(pref,updatepref) {
	var fp = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
	var windowTitle = that.messengerStringBundle.GetStringFromName("SaveAttachment");
    try {
         fp.init(window, windowTitle, Ci.nsIFilePicker.modeGetFolder);
         try {
         	var initialDir = that.preferenceService.prefHasUserValue(pref)? that.preferenceService.getComplexValue(pref, Ci.nsILocalFile) : null;
            if (initialDir) fp.displayDirectory = initialDir;
         } catch (e) {aedump(e,1);}
         var r = fp.show();
         if (r == Ci.nsIFilePicker.returnCancel) return false;
    } catch(e) {aedump(e,0);}
	if (updatepref) that.preferenceService.setComplexValue(pref, Ci.nsILocalFile,fp.file);
    return fp.file;
};
this.getSaveFolder = getSaveFolder;

function getDefaultSaveFolder() {
	if (that.preferenceService.prefHasUserValue("attachmentextractor.defaultsavepath")) {
		return that.preferenceService.getComplexValue("attachmentextractor.defaultsavepath", Ci.nsILocalFile);
	}
	else if (that.preferenceService.getBoolPref("browser.download.useDownloadDir")&&
			 that.preferenceService.prefHasUserValue("browser.download.defaultFolder")) {
		return that.preferenceService.getCharPref("browser.download.defaultFolder");
	} 
	else return that.preferenceService.getCharPref("messenger.save.dir");
};
this.getDefaultSaveFolder = getDefaultSaveFolder;

function getAutoSaveFolder() {
	if (that.preferenceService.prefHasUserValue("attachmentextractor.autoextract.savepath")) {
		return that.preferenceService.getComplexValue("attachmentextractor.autoextract.savepath", Ci.nsILocalFile);
	}
	else return getDefaultSaveFolder();
};
this.getAutoSaveFolder = getAutoSaveFolder;

function getFilenamePattern() {
	var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
	var input = {value: that.getPref("filenamepattern")};
	var check = {value: false};
	if (!prompts.prompt(window,
						that.aeStringBundle.GetStringFromName("FileNamePatternDialogTitle"),
						that.aeStringBundle.GetStringFromName("FileNamePatternDialogMessage"), 
						input, 
						that.aeStringBundle.GetStringFromName("FileNamePatternDialogQuestion"), 
						check)) return null;
	try {
		var fm=new AttachmentFileMaker(null,null,null);
		input.value=fm.validateFilenamePattern(input.value);
	}catch (e) {aedump(e);}
	if (check.value && !input.value) that.setPref("filenamepattern",input.value);
	return input.value;
}

/* check for newer version */
function updatecheck() {
	aedump("//ae: update check starting... ",2);
	if (that.getPref("firstuse")==AE_VERSION) {
		aedump("ae not been updated so abort.\n",2);
		return;
	}
	else that.setPref("firstuse",AE_VERSION);
	aedump("ae has been updated so set pref and show changelog.\n",2);
		
	var name='attachmentextractor';//that.preferenceService.getCharPref("extensions.{35834d20-efdb-4f78-ab77-9635fb4e56c4}.name");
	var lang=that.preferenceService.getCharPref("general.useragent.locale").substring(0,2);
	var url='http://www.eviljeff.com/extensionchecker.php?extension='+name+'&version='+AE_VERSION+'&lang='+lang;
	// 	
	window.openDialog("chrome://attachmentextractor/content/aefirstpopup.xul",
				"_blank", "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar",url);
};

function checkForAutoTag() {
	var ae_tag=that.getPref("autoextract.triggertag");
	if (!that.preferenceService.getPrefType("mailnews.tags."+ae_tag+".tag")) {
		aedump("// AE's auto-tag not found so add it.\n",0);
		var tagService = Cc["@mozilla.org/messenger/tagservice;1"].getService(Ci.nsIMsgTagService);
    	tagService.addTagForKey(ae_tag,'AE AutoExtract','','');
	}
};

function setupListeners() {
	//aedump("{function:AttachmentExtractor.setupListeners}\n",3);
	if (!toggleMRUdone) registerMRUEventListeners();
	try {
		document.getElementById('folderTree').addEventListener('select', updateAECommands, false);
		document.getElementById('threadTree').addEventListener('select', updateAECommands, false);
		//document.getElementById('attachmentListContext').addEventListener('popupshowing',onShowAttachmentContextMenu2,false);
	}
	catch(e){aedump("// setuplisteners failed.\n");}
};

function unSetupListeners() {
	//aedump("{function:AttachmentExtractor.unSetupListeners}\n",3);
	try {
		document.getElementById('folderTree').removeEventListener('select', updateAECommands, false);
		document.getElementById('threadTree').removeEventListener('select', updateAECommands, false);
		//document.getElementById('attachmentListContext').removeEventListener('popupshowing',onShowAttachmentContextMenu2,false);
	}
	catch(e){aedump("// unSetuplisteners failed.\n");}
};

function updateAECommands() {
	var view = GetDBView();
	
	// 	
	var msgcount={};
	var o=new Object();
	if (view) view.getURIsForSelection(o,msgcount);
    
	if (msgcount&&msgcount.value>0) {
		document.getElementById('cmd_ae_extractToDefault').removeAttribute('disabled');
		document.getElementById('cmd_ae_extractToFolder').removeAttribute('disabled');	
		document.getElementById('cmd_ae_extractToFolder_menu').removeAttribute('disabled');	
		document.getElementById('cmd_ae_extractPatternDefault').removeAttribute('disabled');
		document.getElementById('cmd_ae_extractPatternFolder').removeAttribute('disabled');
		document.getElementById('cmd_ae_extractPatternFolder_menu').removeAttribute('disabled');	
		document.getElementById('cmd_ae_justDeleteAttachments').removeAttribute('disabled');
	}
	else {
		document.getElementById('cmd_ae_extractToDefault').setAttribute('disabled','true');
		document.getElementById('cmd_ae_extractToFolder').setAttribute('disabled','true');
		document.getElementById('cmd_ae_extractToFolder_menu').setAttribute('disabled','true');	
		document.getElementById('cmd_ae_extractPatternDefault').setAttribute('disabled','true');
		document.getElementById('cmd_ae_extractPatternFolder').setAttribute('disabled','true');
		document.getElementById('cmd_ae_extractPatternFolder_menu').setAttribute('disabled','true');	
		document.getElementById('cmd_ae_justDeleteAttachments').setAttribute('disabled','true');
	}
	if (view) {
		document.getElementById('cmd_ae_extractAllToDefault').removeAttribute('disabled');
		document.getElementById('cmd_ae_extractAllToFolder').removeAttribute('disabled');		
		document.getElementById('cmd_ae_extractAllToFolder_menu').removeAttribute('disabled');	
		document.getElementById('cmd_ae_extractDeepToDefault').removeAttribute('disabled');
		document.getElementById('cmd_ae_extractDeepToFolder').removeAttribute('disabled');
		document.getElementById('cmd_ae_extractDeepToFolder_menu').removeAttribute('disabled');
		document.getElementById('cmd_ae_justDeleteAllAttachments').removeAttribute('disabled');
	}
	else {
		document.getElementById('cmd_ae_extractAllToDefault').setAttribute('disabled','true');
		document.getElementById('cmd_ae_extractAllToFolder').setAttribute('disabled','true');
		document.getElementById('cmd_ae_extractAllToFolder_menu').setAttribute('disabled','true');	
		document.getElementById('cmd_ae_extractDeepToDefault').setAttribute('disabled','true');
		document.getElementById('cmd_ae_extractDeepToFolder').setAttribute('disabled','true');
		document.getElementById('cmd_ae_extractDeepToFolder_menu').setAttribute('disabled','true');	
		document.getElementById('cmd_ae_justDeleteAllAttachments').setAttribute('disabled','true');
	}
};
this.updateAECommands=updateAECommands;

function registerMRUEventListeners() {
	aedump('{function:AttachmentExtractor.registerMRUEventListeners}\n',2);
	document.getElementById('folderPaneContext').addEventListener('popupshowing',updateMRUVisability,false);
	document.getElementById('threadPaneContext').addEventListener('popupshowing',updateMRUVisability,false);
	clearMRU(that.MRUMAXCOUNT,that.getPref("savepathmru.count")+1); //clear any extra mru slots.
	addToolbarButton();
	toggleMRUdone=true;
};
this.registerMRUEventListeners=registerMRUEventListeners;

function updateMRUVisability(event) {
	var mru=that.getPref("savepathmru");
	
	var children = event.target.childNodes;
	var chattr;
    for (var i=0;i<children.length;i++) {
		if (!children[i]) continue;
		chattr=children[i].getAttribute("ae_mru_menuitem");
		if ((mru && chattr=="MRU") ||
			(!mru&& chattr=="NONMRU") ) {
			children[i].removeAttribute('hidden');
		}
		if ((mru && chattr=="NONMRU") ||
			(!mru&& chattr=="MRU") ) {
			children[i].setAttribute('hidden',true);
		}
	}
};
this.updateMRUVisability=updateMRUVisability;

// 
function updateMRUList(parent) {
	var ps=that.preferenceService2.getBranch("attachmentextractor.");
	if (!ps.getBoolPref("savepathmru")) return;
	
	var children = parent.childNodes;
	var all=(parent.getAttribute("paramAll")=="true");
	var pattern=(parent.getAttribute("paramPattern")=="true");
	var individual=(parent.getAttribute("paramIndividual")=="true");
	
    for (var i=children.length-1;i>=0;i--) {
		if (children[i].getAttribute("ae_mru_menuitem")=="GENERATED") parent.removeChild(children[i]);
	}
	var count=ps.getIntPref("savepathmru.count");
	for (var i=0;i<=count;i++) {
		var menuitem = document.createElement( "menuitem" );
		menuitem.setAttribute( "crop", "center" );
    	if (i<10) menuitem.setAttribute( "accesskey", ""+i );
		menuitem.setAttribute("command","");
		menuitem.setAttribute("ae_mru_menuitem","GENERATED");
		menuitem.setAttribute("oncommand","attachmentextractor.do"+((individual)?"Individual":"")+((pattern)?"Pattern":"")+"Attachmentextraction('"+i+"',"+all+");");	
		if (i==0) menuitem.setAttribute( "label", "(0) "+parent.getAttribute("browseText"));
    	else {
			var pv=(ps.prefHasUserValue("savepathmru."+i))?ps.getComplexValue("savepathmru."+i,Ci.nsILocalFile).path:null;
			if (!pv || pv=="") pv="< ... >";
			if (i<10) pv="("+i+") "+pv;
			menuitem.setAttribute('label',pv);
		}
    	parent.appendChild( menuitem );
	}
	
};
this.updateMRUList=updateMRUList;

function addToMRUList(added) {
	var ps=that.preferenceService2.getBranch("attachmentextractor.");
	if (!ps.getBoolPref("savepathmru")) return added;
	var count=ps.getIntPref("savepathmru.count");
	//prefs.setComplexValue("filename", Components.interfaces.nsILocalFile, file)
	var old=(ps.prefHasUserValue("savepathmru.1"))?ps.getComplexValue("savepathmru.1",Ci.nsILocalFile):null;
	if (old&&added.equals(old)) return added;
	ps.setComplexValue("savepathmru.1",Ci.nsILocalFile,added);
	if (!old) return added;
	var prev=old;
	var i=2;
	for (;i<=count;i++) {
		old=(ps.prefHasUserValue("savepathmru."+i))?ps.getComplexValue("savepathmru."+i,Ci.nsILocalFile):null;
		ps.setComplexValue("savepathmru."+i,Ci.nsILocalFile,prev);
		if (!old || added.equals(old)) break;
		prev=old;
	}
	return added;
};

function useMRU(index) {
	var ps=that.preferenceService2.getBranch("attachmentextractor.");
	var p= (ps.prefHasUserValue("savepathmru."+index))? ps.getComplexValue("savepathmru."+index,Ci.nsILocalFile) : null;
	if (!p) return null;
	return addToMRUList(p);
}

function clearMRU(max,min) {
	if (!min||min<1) min=1;
	if (!max||max>that.MRUMAXCOUNT) max=that.MRUMAXCOUNT;
	var ps=that.preferenceService2.getBranch("attachmentextractor.");
	for (var i=min;i<=max;i++) {
		if (ps.prefHasUserValue("savepathmru."+i)) ps.clearUserPref("savepathmru."+i);
	}
};
this.clearMRU=clearMRU;

function onShowAttachmentContextMenu2(event) {
	//aedump('showing....');
	var saveMenu = document.getElementById('context-saveAttachment');
	var attachmentList = document.getElementById('attachmentList');

	var canOpen = false;
	if (saveMenu.getAttribute('disabled')!="true") {
		//aedump("//");
		for (var i = 0; i < attachmentList.selectedItems.length && !canOpen; i++) {
			canOpen = !attachmentList.selectedItems[i].attachment.isExternalAttachment;
			//aedump(" * "+attachmentList.selectedItems[i].attachment.isExternalAttachment);
		}
		//aedump(" post: "+canOpen+" | "+attachmentList.selectedItems.length+"\n");
	}
	//aedump("//canOpen: "+canOpen+"\n");  //removed 9/3/08
	if (canOpen) 
		 document.getElementById('cmd_ae_extractIndToDefault').removeAttribute('disabled');
	else document.getElementById('cmd_ae_extractIndToDefault').setAttribute('disabled',"true");
	
	if (canOpen) 
		 document.getElementById('menu_ae_extractIndToFolder_attachment'+(that.getPref("savepathmru")?"_menu":"")).removeAttribute('disabled');
	else document.getElementById('menu_ae_extractIndToFolder_attachment'+(that.getPref("savepathmru")?"_menu":"")).setAttribute('disabled',"true");
	
	updateMRUVisability(event);
};
this.onShowAttachmentContextMenu2=onShowAttachmentContextMenu2;

function addToolbarButton()
{
	var mainToolbar = document.getElementById("mail-bar2");
	if (getPref("firstuse.toolbarbutton")) {
		//aedump("//ae: toolbar button existence already checked.\n",3);
		return; //already checked for existence of button. Don't hassle the user any more.
	}
	if (mainToolbar.currentSet.indexOf("attachmentextractor-toolbarbutton") == -1) {
		mainToolbar.insertItem("attachmentextractor-toolbarbutton", document.getElementById('button-delete'), null, false);
		mainToolbar.setAttribute("currentset",mainToolbar.currentSet);
        document.persist("mail-bar2", "currentset");
	} else aedump("//ae: user has already added the toolbar button.\n",3);
    setPref("firstuse.toolbarbutton",true);
} 

function convertPrefs()  {
	var pb=that.preferenceService2.getBranch("attachmentextractor.");
	if (pb.prefHasUserValue("includepatterns")) {
		var newpattern=(pb.getCharPref("includepatterns")=="")? "" : "*."+pb.getCharPref("includepatterns").split(',').join(';*.');
		pb.setCharPref("includepatterns4",newpattern);
		pb.clearUserPref("includepatterns");
	} 
	if (pb.prefHasUserValue("includepatterns2")) {
		var newpattern=pb.getCharPref("includepatterns2").replace(",",";","g");
		pb.setCharPref("includepatterns4",newpattern);
		pb.clearUserPref("includepatterns2");
	}
	if (pb.prefHasUserValue("includepatterns3")) {
		var patterns=pb.getCharPref("includepatterns3").split(';');
		for (var i=0;i++;i<patterns.length) if (patterns[i].indexOf("*")==-1) patterns[i]="*"+patterns[i];
		pb.setCharPref("includepatterns4",patterns.join(";"));
		pb.clearUserPref("includepatterns3");
	}
	if (pb.prefHasUserValue("excludepatterns")) {
		var newpattern=(pb.getCharPref("excludepatterns")=="")? "" : "*."+pb.getCharPref("excludepatterns").split(',').join(';*.');
		pb.setCharPref("excludepatterns4",newpattern);
		pb.clearUserPref("excludepatterns");
	} 
	if (pb.prefHasUserValue("excludepatterns2")) {
		var newpattern=pb.getCharPref("excludepatterns2").replace(",",";","g");
		pb.setCharPref("excludepatterns4",newpattern);
		pb.clearUserPref("excludepatterns2");
	}
	if (pb.prefHasUserValue("excludepatterns3")) {
		var patterns=pb.getCharPref("excludepatterns3").split(';');
		for (var i=0;i++;i<patterns.length) if (patterns[i].indexOf("*")==-1) patterns[i]="*"+patterns[i];
		pb.setCharPref("excludepatterns4",patterns.join(";"));
		pb.clearUserPref("excludepatterns3");
	}
	if (pb.prefHasUserValue("aedetacher")) {
		if (pb.getBoolPref("aedetacher")) {
			pb.setIntPref("actionafterextract.detach.mode",(!pb.prefHasUserValue("aedetacher.delete"))?1:2);
		}
		pb.clearUserPref("aedetacher");	
	}
	if (pb.prefHasUserValue("extract")) {
		if (!pb.getBoolPref("extract")) {
			pb.setIntPref("extract.mode",-1);
		}
		pb.clearUserPref("extract");	
	}

};

// 
};
if (!attachmentextractor) var attachmentextractor=new AttachmentExtractor();

var aefolderListener = {
  OnItemAdded: function(parentItem, item) {
	if (!attachmentextractor.getPref("autoextract")) return;
						   
	var mail;
	try{
		mail=item.QueryInterface(Components.interfaces.nsIMsgDBHdr);}
	catch (e) {return;}
    var folder=parentItem.QueryInterface(Components.interfaces.nsIMsgFolder); 
	if (!(!mail.isRead && (mail.flags & 0x10000))) {
		//aedump("// not a new mail so don't extract\n",3);
		return; 
	}
	aedump("{function:aefolderlistener.onItemAdded("+folder.prettyName+","+mail.subject+")}\n",2);
	if (!folder.getMsgDatabase(null).HasAttachments(mail.messageKey)) {
		aedump("// message has no attachments so ignoring.\n",3);
		return; 
	}
	var tagsArray= mail.getStringProperty("keywords").split(" ");
	var triggerTag=attachmentextractor.getPref("autoextract.triggertag");
	//aedump("[tags array: "+mail.getStringProperty("keywords")+"]\n",0);
	//aedump("[trigger tag: "+attachmentextractor.getPref("autoextract.triggertag")+"]\n",0);
	if (attachmentextractor.getPref("autoextract.ontriggeronly") && (tagsArray.indexOf(triggerTag)==-1) ) {
		aedump("// only tagged emails and tag doesn't match\n",3);
		return;
	}
	if (attachmentextractor.getPref("autoextract.waitforall")) {
		attachmentextractor.autoUris.push(mail.folder.getUriForMsg(mail));
	}
	else attachmentextractor.doSingleBackgroundAttachmentextraction(mail.folder.getUriForMsg(mail));
  },
 
  OnItemPropertyChanged: function(item, property, oldValue, newValue) {
	  var folder;
	  try {folder=item.QueryInterface(Components.interfaces.nsIMsgFolder); }catch(e) {return;}
	  aedump("{function:OnItemPropertyChanged("+folder.prettyName+","+property+oldValue+","+newValue+")}\n",2);
	  if (newValue>oldValue && attachmentextractor.autoUris.length!=0) {
		attachmentextractor.doBackgroundAttachmentextraction();  
	  }
  },
  
  OnItemRemoved: function(parentItem, item) {aedump("{function:OnItemRemoved("+argexpand(arguments)+")}\n",4);},
  OnItemIntPropertyChanged: function(item, property, oldValue, newValue) {aedump("{function:OnItemIntPropertyChanged("+argexpand(arguments)+")}\n",4);},
  OnItemBoolPropertyChanged: function(item, property, oldValue, newValue) {aedump("{function:OnItemBoolPropertyChanged("+argexpand(arguments)+")}\n",4);},
  OnItemUnicharPropertyChanged: function(item, property, oldValue, newValue){aedump("{function:OnItemUnicharPropertyChanged("+argexpand(arguments)+")}\n",4);},
  OnItemPropertyFlagChanged: function(item, property, oldFlag, newFlag) {aedump("{function:OnItemPropertyFlagChanged("+argexpand(arguments)+")}\n",4);},
  OnItemEvent: function(folder, event) {aedump("{function:OnItemEvent("+argexpand(arguments)+")}\n",4);}   
};

function AEPrefObserver() {
  function observe(subject ,topic , data) {
	//aedump("// "+topic+","+data+"\n");
	if (topic!="nsPref:changed") return;
	if (data=="attachmentextractor.savepathmru"||data=="attachmentextractor.savepathmru.count") {
	  if (attachmentextractor.getPref("savepathmru")) {
		attachmentextractor.clearMRU(that.MRUMAXCOUNT,attachmentextractor.getPref("savepathmru.count")+1); //clear any extra mru slots.
	  }
	}
  }
  this.observe=observe;

  /* should support all interfaces we support */
  this.QueryInterface= function (iid)
  {
    if (iid.equals(Components.interfaces.nsIObserver) ||
        iid.equals(Components.interfaces.nsISupports))
      return this;
    throw Components.results.NS_NOINTERFACE;
  };
}

// 
var aedebug=false;
var aedebugFile=null;
try{
	aedebug=Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefBranch).getBoolPref("attachmentextractor.debug");
	if (aedebug) {
		aedebug= Components.classes['@mozilla.org/network/file-output-stream;1'].
						createInstance(Components.interfaces.nsIFileOutputStream);
		aedebugFile=Components.classes["@mozilla.org/file/directory_service;1"].
						getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
		aedebugFile.append('aedebug.txt');
	}
}catch (e) {}

var argexpand = (aedebug)? function (args) {
	var str="";
	for(var i=0;i<args.length;i++) {
		if (i>0) str+=",";
		str+=args[i].toString();
	}
	return str;
} : function () {return "";};

var aedump = (aedebug)? function (){
	var loglevel=3;
	var showComponents=new Array("","aemessenger");
	
	var errorlevel=0;
	if (arguments.length>1) errorlevel=arguments[1];
	if (arguments.length>2) {
		var component=arguments[2];
		if (showComponents.indexOf(component)==-1) return;
	}
	if (errorlevel<=loglevel) {
		try{ 
			var str=(arguments[0]+"").replace("\n","\r\n");
			aedebug.init(aedebugFile, 0x02 | 0x10, 0664, 0);
			aedebug.write(str, str.length);
			aedebug.close(); 
		} catch(e) {dump("NOT LOGGED: ");}
		dump(arguments[0]);
	}
} : function () {};

var createaedumpfile = (aedebug)? function () {
	try {
		aedebug.init(aedebugFile, 0x02 | 0x08 | 0x20, 0664, 0);
		var str="//log start \r\n";
		aedebug.write(str, str.length);
		
		/* following enabled by build script in xpi only: */		
		str = "AE Set Preferences: {"+printPrefValues("attachmentextractor.",false,/.*(patterns4$|[^0-9]$)/i)+"}\r\n";
		aedebug.write(str, str.length);
		 /* end */
		
		aedebug.close(); 
	}
	catch(e) {dump(e);}
} : function() {};

var printPrefValues = (aedebug)? function (branch,includeDefault,includereg) {
	branch=Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch).getBranch(branch);
	var children=branch.getChildList("", {});
	var out=null;
	for (var i=0;i<children.length;i++) {
		if (includereg.test(children[i]) && 
			(includeDefault || branch.prefHasUserValue(children[i]))) {
			if (!out) out=""; else out+=", \r\n";
			var val;
			switch (branch.getPrefType(children[i])) {
				case branch.PREF_BOOL  : val= branch.getBoolPref(children[i]); break;
				case branch.PREF_INT   : val= branch.getIntPref(children[i]); break;
				case branch.PREF_STRING: val= '"'+branch.getCharPref(children[i])+'"'; break;
				default :
			}
			out+=children[i]+":"+val;
		}
	}
	return out;
} : function() {};
